"use strict";

var _require = require('datocms-client'),
    SiteClient = _require.SiteClient,
    Loader = _require.Loader;

var CLIENT_HEADERS = {
  'X-Reason': 'dump',
  'X-SSG': 'gatsby'
};
var GATSBY_CLOUD = process.env.GATSBY_CLOUD;
var GATSBY_EXECUTING_COMMAND = process.env.gatsby_executing_command;
var clients = {};
var loaders = {};

function getClient(options) {
  var apiToken = options.apiToken,
      apiUrl = options.apiUrl,
      environment = options.environment,
      logApiCalls = options.logApiCalls;
  var key = JSON.stringify({
    apiToken: apiToken,
    apiUrl: apiUrl,
    environment: environment,
    logApiCalls: logApiCalls
  });

  if (clients[key]) {
    return clients[key];
  }

  var clientOptions = {
    headers: CLIENT_HEADERS
  };

  if (options.environment) {
    clientOptions.environment = environment;
  }

  if (options.baseUrl) {
    clientOptions.baseUrl = apiUrl;
  }

  if (options.logApiCalls) {
    clientOptions.logApiCalls = logApiCalls;
  }

  var client = new SiteClient(apiToken, clientOptions);
  clients[key] = client;
  return client;
}

function getLoader(options) {
  var apiToken = options.apiToken,
      apiUrl = options.apiUrl,
      previewMode = options.previewMode,
      environment = options.environment,
      pageSize = options.pageSize,
      logApiCalls = options.logApiCalls;
  var key = JSON.stringify({
    apiToken: apiToken,
    apiUrl: apiUrl,
    previewMode: previewMode,
    environment: environment,
    pageSize: pageSize,
    logApiCalls: logApiCalls
  });

  if (loaders[key]) {
    return loaders[key];
  }

  var loader = new Loader(getClient({
    apiToken: apiToken,
    apiUrl: apiUrl,
    environment: environment,
    logApiCalls: logApiCalls
  }), GATSBY_CLOUD && GATSBY_EXECUTING_COMMAND === 'develop' || previewMode, environment, {
    pageSize: pageSize
  });
  loaders[key] = loader;
  return loader;
}

module.exports = {
  getClient: getClient,
  getLoader: getLoader
};